#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gsl_rng.h>
#include <stdio.h>
#include <math.h>
#include <glib.h>
#include "moiss.h"
#include "output.h"
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include "bto.h"
#include <errno.h>
#include <string.h>
#include "proto.h"
#include "bto.h"
#include "init.h"
#include "rand.h"

#include "stats.h"

double variational_M(ensemble * e, GList * stats, int num_steps)
{
  
  int i;
  int vstp, Nw;
    double   Accept,A;
    double del1;
    double psi1;
    double vnew;
    double vvold[ncoord];
    double avg,vave;
    int bol_wf = 0; /* do normalize?*/
    bol_wf = stats_check_wavef(stats);
    
    del1=0.0;
    Accept=0;
    vave=0.0;
    
    for (vstp = 0; vstp < num_steps; vstp++) 
      {
	for (Nw = 0; Nw < initial_walkers; Nw++) 
	  {
	    psi1=e->walkers[Nw].Psi;
	    for (i = 0; i < ncoord; i++) 
	      {
		vvold[i] = e->walkers[Nw].R[i];
		e->walkers[Nw].R[i]+= normal_dist() * diff_sigma;
	      }
	    
	    vnew = (*V2)(&e->walkers[Nw]);  
	    
	    (*Derivs)(&e->walkers[Nw]);
	    A =(e->walkers[Nw].Psi * e->walkers[Nw].Psi)/(psi1 * psi1);
	    	    
	    if (A > gsl_rng_uniform(rng)) 

	      {
		Accept++;
		e->walkers[Nw].Egy_Lcl = 
		  -0.5 * e->walkers[Nw].del2 / e->walkers[Nw].Psi + vnew;
	      }
	    else
	      {
		for (i = 0; i < ncoord; i++) 
		  e->walkers[Nw].R[i] = vvold[i];
		e->walkers[Nw].Psi = psi1;
	     
	      }
	    vave += e->walkers[Nw].Egy_Lcl;

	    
	 	  
	  }
	if(bol_wf)
	  stats_histogram(e);
      }
    
    avg = vave / initial_walkers / num_steps;
    return avg;
   // energy_vs_time[block_count] = avg;
}

double variational_FPF(ensemble * e, GList * stats, int num_steps)
{
    /*  Variational Monte Carlo based in the Fokker-Planck formalism */


    register int i, k = 0;
    int vstp, Nw;
    double Accept = 0, A, vnew;
    double Q, oldfqf[ncoord], temp1, vvold[ncoord];
    double psi1;
    double engy1;
    double vave=0,avg;
     int bol_wf = 0; /* do normalize?*/
    bol_wf = stats_check_wavef(stats);
    temp1 = D * dt / 2.0;
   
    for (vstp = 0; vstp < num_steps; vstp++) {
      for (Nw = 0; Nw < initial_walkers; Nw++) {


	psi1 = e->walkers[Nw].Psi;
	engy1 = e->walkers[Nw].Egy_Lcl;
	for (i = 0; i < ncoord; i++) 
	  {
	    oldfqf[i]=e->walkers[Nw].Force[i];
	    vvold[i]=e->walkers[Nw].R[i];
	    e->walkers[Nw].R[i] += normal_dist() * diff_sigma  + e->walkers[Nw].Force[i] * Ddt;
	  }
	vnew = (*V2)(&e->walkers[Nw]);
	(*Derivs)(&e->walkers[Nw]);
	for (i = 0; i < ncoord; i++)
	e->walkers[Nw].Force[i]   = 
	  2 * e->walkers[Nw].grad[i] / e->walkers[Nw].Psi;
	Q = 0.0;
	for (i = 0; i < ncoord; i++)

	    Q += (oldfqf[i] + e->walkers[Nw].Force[i]) *
	      (temp1 * (oldfqf[i] - e->walkers[Nw].Force[i]) -
	       ((e->walkers[Nw].R[i]) - vvold[i]));

	if(Q<-1300) {
	 
	  //  printf("%f %d %d \n",Q,Nw,vstp);
	  A=0.0;

	}
	else
	  A = ((e->walkers[Nw].Psi * e->walkers[Nw].Psi) / (psi1 * psi1)) *exp(0.5 * Q);
	if (A > gsl_rng_uniform(rng)) 
	  {
	    Accept++;
	    e->walkers[Nw].Egy_Lcl = -0.5 * e->walkers[Nw].del2 / e->walkers[Nw].Psi + vnew;
	  }
	else
	  {
	    for (i = 0; i < ncoord ; i++) 
	      {
		  e->walkers[Nw].R[i] = vvold[i];
		  e->walkers[Nw].Force[i] = oldfqf[k];
	      }
	    e->walkers[Nw].Psi = psi1;
	  

  }


	vave += e->walkers[Nw].Egy_Lcl;
	
      }
      if(bol_wf)
	stats_histogram(e);
    }
    avg = vave / initial_walkers / num_steps;
    return avg;
//    energy_vs_time[block_count] = avg;
   // return (float) (Accept / (float)initial_walkers / (float)num_steps);
  //  printf("** Aceptad %f \n ",Accept);
}
