#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gsl_rng.h>
#include <math.h>
#include <glib.h>
#include "stdio.h"
#include "moiss.h"
#include "../gmoiss-src/gmoiss.h"
#include "walk.h"
#include "output.h"
#include "gui.h"
#include "method.h"
#include "rand.h"
#include "stats.h" 

void walk_block(ensemble * e, MonteCarlo * mc)
{ 
  double bEavg, bEsqsum, bEstddev;
  double bEsum, bEsumsq;
  double porctg, regi;
  int bol_avg= OFF; /* do averages? */
  int bol_EvsT= OFF;
  int block;
  int num_blocks; /* number of blocks from the Monte Carlo */
  int num_steps;
  double Eb;
  GList * stats_list;
  bEsum = 0;
  bEsumsq = 0;
  num_blocks = mc->blocks;
  num_steps = mc->iterations;
  regi = (double) num_blocks / 10.0;
  
  stats_list = mc->stats;
  //  bol_avg = stats_check_average(stats_list);
  bol_EvsT = stats_check(stats_list,ENERGY_PLOT);
  printf("%d \n",bol_EvsT);
  printf("Blocks  Steps     Energy       %% \n");
  
  for (block = 1; block < num_blocks+1; block++) 
    {
      block_count++;
      Eb=(*Walk) (e, stats_list, num_steps);
      porctg = 100.0 * block / num_blocks;
      if(num_blocks<10)
	printf(" %3d   %5d   %.10f   %2.1f  \n", block, block * num_steps
	       , Eb, porctg);
      else
	if ((fmod((double) block, regi)) == 0)
	  printf(" %3d   %5d   %.10f   %2.1f \n", block, block * num_steps
		 , Eb, porctg);
      
      if (bol_EvsT==1)
	energy_vs_time[block_count]=Eb;
      
      if (bol_avg==1) {
	bEsum += Eb;//energy_vs_time[block_count];
	bEsumsq +=Eb*Eb; //energy_vs_time[block_count] * energy_vs_time[block_count];
      }
      
    }
  bEavg = bEsum / num_blocks;
  E_Trial=bEavg;
  if (bol_avg==1) {
    bEsqsum = bEavg * bEavg * num_blocks;
    
    if ((bEsumsq - bEsqsum) < 0) {
	bEsumsq = bEsqsum;
	printf("Negative difference. Cannot calculate std. deviation properly.\n");
    }
    bEstddev = sqrt((bEsumsq - bEsqsum) / (num_blocks - 1));
    printf("Average Energy:  %.12f  (+/-) %.12f\n", bEavg, bEstddev);
    
  }
  /*autocor(num_blocks); */
}

double walk_with_branch_IS(ensemble * e, GList * stats, int num_steps)
{
  double vsum = 0;
  double vave = 0;
  double sum = 0, avg;
  int i, j = 0, k,  nwalker;
  int num_branches;
  double a = 0.1;		/* Factor for acceptance, may be varied */
  double Rold[ncoord],fqfold[ncoord],psi1;
  double vnew,Q,temp1,A;
  int Accept=0,tot_cor=0;
  double dsqa=0,dsq=0,dr,desp;
  double oldpotential;
  int bol_wf = OFF; /* do normalize?*/
  bol_wf = stats_check_wavef(stats);
  
    nwalker = num_walkers; /* global variable, dissapear why mirror it in a local one?	 FIXME */
    
    temp1 = D * dt / 2.0;
    for (i = 0; i < num_steps; i++) {
      for (j = nwalker-1; j >= 0; j--) {
	oldpotential = e->walkers[j].Egy_Lcl;
	dr=0;
	
	psi1=e->walkers[j].Psi;
	for (k = 0; k < ncoord; k++) 
	  {
	    desp=normal_dist() * diff_sigma + e->walkers[j].Force[k] * Ddt;
	    Rold[k]=e->walkers[j].R[k];
	    e->walkers[j].R[k]+=desp;
	    dr+=P2(desp);
	  }
	dsq+=dr;
	
	vnew = (*V2) (&e->walkers[j]);
	(*Derivs)(&e->walkers[j]);
	for (k = 0; k < ncoord; k++)
	  {
	    fqfold[k] = e->walkers[j].Force[k];
	    e->walkers[j].Force[k]=2 * e->walkers[j].grad[k] / e->walkers[j].Psi;
	  }
	
	Q = 0.0;
	for (k = 0; k < ncoord; k++)
	  Q +=(fqfold[k] + e->walkers[j].Force[k])*
	    (temp1 * (fqfold[k] - e->walkers[j].Force[k]) -
	     (e->walkers[j].R[k] - Rold[k]));
	
	if(Q<-1300) {
	  
	  //  printf("%f %d %d \n",Q,Nw,vstp);
	  A=0.0;
	  
	}
	else
	  {
	    A =((e->walkers[j].Psi * e->walkers[j].Psi)/(psi1 * psi1))* exp(0.5 * Q);
	    tot_cor++;
	  }
	if (A > gsl_rng_uniform(rng)) 
	  {
	    Accept++;
	    dsqa+=dr;
	    e->walkers[j].Egy_Lcl = vnew= -0.5 * 
	      e->walkers[j].del2 / e->walkers[j].Psi + vnew;
	
	    num_branches =  exp(( - (0.5*(vnew + oldpotential))  
				  + E_Trial) * teff)+ gsl_rng_uniform(rng);
	    switch (num_branches) {
	    case 0:
	      copy_walker(&(e->walkers[nwalker]), &(e->walkers[j]));
	      nwalker--;
	      break;
	    case 1:
	      vsum += vnew;
	      break;
	    default:
	      if (num_branches > max_branch)
		num_branches = max_branch;
	      for (k = 1; k < num_branches; k++) 
		{
		  nwalker++;
		  copy_walker(&(e->walkers[j]), &(e->walkers[nwalker]));
		  vsum += vnew;
		}
	      break;
	    }
	  }
	else
	  {
	    e->walkers[j].Psi = psi1;
	    for (k = 0; k < ncoord; k++)
	      {
		e->walkers[j].Force[k]=fqfold[k];
		e->walkers[j].R[k]=Rold[k];
	      }	
	    vsum += vnew;
	  }
	//	printf("%d",(0.5*(potential + oldpotential))  + E_Trial*teff);
	
	
      }
      
      if(nwalker<=1)
	g_error("Se murieron todos los walkers");
      //printf("** %d %f %f %f \n",nwalker,E_Trial,vsum/nwalker,potential);
      vave = vsum / nwalker;
      do_stats(e, stats);
      sum += vave;
      E_Trial = vave 
	-a * (nwalker - initial_walkers) / (initial_walkers * dt);
      
      vsum = 0;
      if(bol_wf)
        stats_histogram(e);
      
    }
    teff=dt*(dsqa/Accept)/(dsq/num_steps);
    avg = sum / num_steps;
    //energy_vs_time[block_count] = avg;
    teff=dt*(dsqa/(double)Accept)/(dsq/(double)tot_cor);
    num_walkers=nwalker; /* kinda lame */
    return avg;
}


double walk_with_branch(ensemble * e, GList * stats, int num_steps)
{
  double vsum = 0;
  double vave = 0;
  double sum = 0, avg;
  int i, j = 0, k,nwalker;
  double potential,oldpotential;
  int num_branches;
  double a = 0.1;		/* Factor for acceptance, may be varied */
  int bol_wf = OFF; /* do normalize?*/
  bol_wf = stats_check_wavef(stats);
  
  nwalker=num_walkers;
  
  for (i = 0; i < num_steps; i++) {
    for (j = nwalker-1; j >= 0; j--) {
      oldpotential = e->walkers[j].Egy_Lcl;
      for (k = 0; k < ncoord; k++)
	  e->walkers[j].R[k] += normal_dist() * diff_sigma;

      
      potential = (*V2) (&e->walkers[j]);
      e->walkers[j].Egy_Lcl=potential;
      
      num_branches = exp(-((0.5 * (potential + oldpotential)) - E_Trial) * dt)
	
	+ gsl_rng_uniform(rng);
      
	switch (num_branches) {
	case 0:
	  copy_walker(&(e->walkers[nwalker]), &(e->walkers[j]));
	  nwalker--;
	  break;
	case 1:
	  vsum += potential;
	  break;
	default:
	  if (num_branches > max_branch)
	    num_branches = max_branch;
	  for (k = 1; k < num_branches; k++) {
	    nwalker++;
	    copy_walker(&e->walkers[j], &e->walkers[nwalker]);
	    vsum += potential;
	  }
	  break;
	}
    }
    
    vave = vsum /(double) nwalker;
    do_stats(e, stats);
    sum += vave;
    E_Trial = vave 
      -a * (nwalker - initial_walkers) / (initial_walkers * dt);
    //      printf("*** %f %f %d \n",E_Trial,vave,nwalker);
    vsum = 0;
    if(bol_wf)
      stats_histogram(e);
  }
  avg = sum / num_steps;
  //energy_vs_time[block_count] = avg;
  num_walkers=nwalker;
  return avg;
}

double walk_pure_1(ensemble * e, GList * stats, int num_steps)
{
  return .20;
}

/*
  void walk_pure_1(ensemble * e,int bol_normalize)
  {
  double W = 0, E_mix = 0, ateff = 0, average = 0;
  int i, j = 0, Block;
  double vpotential[initial_walkers];
  double GB,oldpotential,potential;
  int bol_wf = OFF;  
  bol_wf = stats_check_wavef(stats);
  num_walkers = initial_walkers;
  
  for (i = 0; i < num_steps; i++) {
  teff = 0;
  for (j = 0; j < initial_walkers; j++) {
  oldpotential = e->walkers[j].Egy_Lcl;
  (* G_Diff)(&(e->walkers[j]));
  vpotential[j] = 0.5 * (e->walkers[j].Egy_Lcl + oldpotential);
  }
  
  ateff = (double) teff / initial_walkers;
  for (j = 0; j < initial_walkers; j++) {
  GB = exp(-(vpotential[j] - E_Trial) * ateff);
  W += GB;
  E_mix_b += GB * e->walkers[j].Egy_Lcl;
  }
  printf("W= %f \n", W);
  E_mix_PI *= exp(-ateff * E_Trial);
  E_mix = E_mix_PI * E_mix_b / (E_mix_PI * W);
  printf("%f %f %f %f %f \n", ateff, E_Trial, E_mix_PI, E_mix_b, E_mix);
  printf("E= %f \n", E_mix);
  E_Trial = E_mix - ((dt / ateff) * log(W / initial_walkers));
  W = 0;
  E_mix_b = 0;
  average += E_mix;
  energy_vs_time[Block * num_steps + i] = E_mix;
  if(bol_wf)
  stats_histogram(e);
  }
  eblk[Block] = average / num_steps;
  }
*/

